# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Iterable
from datetime import datetime
from typing_extensions import Literal

import httpx

from ..types import pricing_list_params, pricing_create_params, pricing_update_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursor, AsyncCursor
from .._base_client import AsyncPaginator, make_request_options
from ..types.pricing_response import PricingResponse
from ..types.shared_params.pricing_band import PricingBand

__all__ = ["PricingsResource", "AsyncPricingsResource"]


class PricingsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PricingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return PricingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PricingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return PricingsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        org_id: str | None = None,
        pricing_bands: Iterable[PricingBand],
        start_date: Union[str, datetime],
        accounting_product_id: str | NotGiven = NOT_GIVEN,
        aggregation_id: str | NotGiven = NOT_GIVEN,
        code: str | NotGiven = NOT_GIVEN,
        compound_aggregation_id: str | NotGiven = NOT_GIVEN,
        cumulative: bool | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        end_date: Union[str, datetime] | NotGiven = NOT_GIVEN,
        minimum_spend: float | NotGiven = NOT_GIVEN,
        minimum_spend_bill_in_advance: bool | NotGiven = NOT_GIVEN,
        minimum_spend_description: str | NotGiven = NOT_GIVEN,
        overage_pricing_bands: Iterable[PricingBand] | NotGiven = NOT_GIVEN,
        plan_id: str | NotGiven = NOT_GIVEN,
        plan_template_id: str | NotGiven = NOT_GIVEN,
        segment: Dict[str, str] | NotGiven = NOT_GIVEN,
        tiers_span_plan: bool | NotGiven = NOT_GIVEN,
        type: Literal["DEBIT", "PRODUCT_CREDIT", "GLOBAL_CREDIT"] | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PricingResponse:
        """
        Create a new Pricing.

        **Note:** Either `planId` or `planTemplateId` request parameters are required
        for this call to be valid. If you omit both, then you will receive a validation
        error.

        Args:
          pricing_bands

          start_date: The start date _(in ISO-8601 format)_ for when the Pricing starts to be active
              for the Plan of Plan Template._(Required)_

          accounting_product_id: Optional Product ID this Pricing should be attributed to for accounting purposes

          aggregation_id: UUID of the Aggregation used to create the Pricing. Use this when creating a
              Pricing for a segmented aggregation.

          code: Unique short code for the Pricing.

          compound_aggregation_id: UUID of the Compound Aggregation used to create the Pricing.

          cumulative: Controls whether or not charge rates under a set of pricing bands configured for
              a Pricing are applied according to each separate band or at the highest band
              reached.

              _(Optional)_. The default value is **FALSE**.

              - When TRUE, at billing charge rates are applied according to each separate
                band.

              - When FALSE, at billing charge rates are applied according to highest band
                reached.

              **NOTE:** Use the `cumulative` parameter to create the type of Pricing you
              require. For example, for Tiered Pricing set to **TRUE**; for Volume Pricing,
              set to **FALSE**.

          description: Displayed on Bill line items.

          end_date: The end date _(in ISO-8601 format)_ for when the Pricing ceases to be active for
              the Plan or Plan Template.

              _(Optional)_ If not specified, the Pricing remains active indefinitely.

          minimum_spend: The minimum spend amount per billing cycle for end customer Accounts on a Plan
              to which the Pricing is applied.

          minimum_spend_bill_in_advance: The default value is **FALSE**.

              - When TRUE, minimum spend is billed at the start of each billing period.

              - When FALSE, minimum spend is billed at the end of each billing period.

              _(Optional)_. Overrides the setting at Organization level for minimum spend
              billing in arrears/in advance.

          minimum_spend_description: Minimum spend description _(displayed on the bill line item)_.

          overage_pricing_bands: Specify Prepayment/Balance overage pricing in pricing bands for the case of a
              **Tiered** pricing structure.

          plan_id: UUID of the Plan the Pricing is created for.

          plan_template_id: UUID of the Plan Template the Pricing is created for.

          segment: Specifies the segment value which you are defining a Pricing for using this
              call:

              - For each segment value defined on a Segmented Aggregation you must create a
                separate Pricing and use the appropriate `aggregationId` parameter for the
                call.
              - If you specify a segment value that has not been defined for the Aggregation,
                you'll receive an error.
              - If you've defined segment values for the Aggregation using a single wildcard
                or multiple wildcards, you can create Pricing for these wildcard segment
                values also.

              For more details on creating Pricings for segment values on a Segmented
              Aggregation using this call, together with some examples, see the
              [Using API Call to Create Segmented Pricings](https://www.m3ter.com/docs/guides/plans-and-pricing/pricing-plans/pricing-plans-using-segmented-aggregations#using-api-call-to-create-a-segmented-pricing)
              in our User Documentation.

          tiers_span_plan: The default value is **FALSE**.

              - If TRUE, usage accumulates over the entire period the priced Plan is active
                for the account, and is not reset for pricing band rates at the start of each
                billing period.

              - If FALSE, usage does not accumulate, and is reset for pricing bands at the
                start of each billing period.

          type: - **DEBIT**. Default setting. The amount calculated using the Pricing is added
                to the bill as a debit.

              - **PRODUCT_CREDIT**. The amount calculated using the Pricing is added to the
                bill as a credit _(negative amount)_. To prevent negative billing, the bill
                will be capped at the total of other line items for the same Product.

              - **GLOBAL_CREDIT**. The amount calculated using the Pricing is added to the
                bill as a credit _(negative amount)_. To prevent negative billing, the bill
                will be capped at the total of other line items for the entire bill, which
                might include other Products the Account consumes.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/pricings",
            body=maybe_transform(
                {
                    "pricing_bands": pricing_bands,
                    "start_date": start_date,
                    "accounting_product_id": accounting_product_id,
                    "aggregation_id": aggregation_id,
                    "code": code,
                    "compound_aggregation_id": compound_aggregation_id,
                    "cumulative": cumulative,
                    "description": description,
                    "end_date": end_date,
                    "minimum_spend": minimum_spend,
                    "minimum_spend_bill_in_advance": minimum_spend_bill_in_advance,
                    "minimum_spend_description": minimum_spend_description,
                    "overage_pricing_bands": overage_pricing_bands,
                    "plan_id": plan_id,
                    "plan_template_id": plan_template_id,
                    "segment": segment,
                    "tiers_span_plan": tiers_span_plan,
                    "type": type,
                    "version": version,
                },
                pricing_create_params.PricingCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PricingResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PricingResponse:
        """
        Retrieve the Pricing with the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/pricings/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PricingResponse,
        )

    def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        pricing_bands: Iterable[PricingBand],
        start_date: Union[str, datetime],
        accounting_product_id: str | NotGiven = NOT_GIVEN,
        aggregation_id: str | NotGiven = NOT_GIVEN,
        code: str | NotGiven = NOT_GIVEN,
        compound_aggregation_id: str | NotGiven = NOT_GIVEN,
        cumulative: bool | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        end_date: Union[str, datetime] | NotGiven = NOT_GIVEN,
        minimum_spend: float | NotGiven = NOT_GIVEN,
        minimum_spend_bill_in_advance: bool | NotGiven = NOT_GIVEN,
        minimum_spend_description: str | NotGiven = NOT_GIVEN,
        overage_pricing_bands: Iterable[PricingBand] | NotGiven = NOT_GIVEN,
        plan_id: str | NotGiven = NOT_GIVEN,
        plan_template_id: str | NotGiven = NOT_GIVEN,
        segment: Dict[str, str] | NotGiven = NOT_GIVEN,
        tiers_span_plan: bool | NotGiven = NOT_GIVEN,
        type: Literal["DEBIT", "PRODUCT_CREDIT", "GLOBAL_CREDIT"] | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PricingResponse:
        """
        Update Pricing for the given UUID.

        **Note:** Either `planId` or `planTemplateId` request parameters are required
        for this call to be valid. If you omit both, then you will receive a validation
        error.

        Args:
          pricing_bands

          start_date: The start date _(in ISO-8601 format)_ for when the Pricing starts to be active
              for the Plan of Plan Template._(Required)_

          accounting_product_id: Optional Product ID this Pricing should be attributed to for accounting purposes

          aggregation_id: UUID of the Aggregation used to create the Pricing. Use this when creating a
              Pricing for a segmented aggregation.

          code: Unique short code for the Pricing.

          compound_aggregation_id: UUID of the Compound Aggregation used to create the Pricing.

          cumulative: Controls whether or not charge rates under a set of pricing bands configured for
              a Pricing are applied according to each separate band or at the highest band
              reached.

              _(Optional)_. The default value is **FALSE**.

              - When TRUE, at billing charge rates are applied according to each separate
                band.

              - When FALSE, at billing charge rates are applied according to highest band
                reached.

              **NOTE:** Use the `cumulative` parameter to create the type of Pricing you
              require. For example, for Tiered Pricing set to **TRUE**; for Volume Pricing,
              set to **FALSE**.

          description: Displayed on Bill line items.

          end_date: The end date _(in ISO-8601 format)_ for when the Pricing ceases to be active for
              the Plan or Plan Template.

              _(Optional)_ If not specified, the Pricing remains active indefinitely.

          minimum_spend: The minimum spend amount per billing cycle for end customer Accounts on a Plan
              to which the Pricing is applied.

          minimum_spend_bill_in_advance: The default value is **FALSE**.

              - When TRUE, minimum spend is billed at the start of each billing period.

              - When FALSE, minimum spend is billed at the end of each billing period.

              _(Optional)_. Overrides the setting at Organization level for minimum spend
              billing in arrears/in advance.

          minimum_spend_description: Minimum spend description _(displayed on the bill line item)_.

          overage_pricing_bands: Specify Prepayment/Balance overage pricing in pricing bands for the case of a
              **Tiered** pricing structure.

          plan_id: UUID of the Plan the Pricing is created for.

          plan_template_id: UUID of the Plan Template the Pricing is created for.

          segment: Specifies the segment value which you are defining a Pricing for using this
              call:

              - For each segment value defined on a Segmented Aggregation you must create a
                separate Pricing and use the appropriate `aggregationId` parameter for the
                call.
              - If you specify a segment value that has not been defined for the Aggregation,
                you'll receive an error.
              - If you've defined segment values for the Aggregation using a single wildcard
                or multiple wildcards, you can create Pricing for these wildcard segment
                values also.

              For more details on creating Pricings for segment values on a Segmented
              Aggregation using this call, together with some examples, see the
              [Using API Call to Create Segmented Pricings](https://www.m3ter.com/docs/guides/plans-and-pricing/pricing-plans/pricing-plans-using-segmented-aggregations#using-api-call-to-create-a-segmented-pricing)
              in our User Documentation.

          tiers_span_plan: The default value is **FALSE**.

              - If TRUE, usage accumulates over the entire period the priced Plan is active
                for the account, and is not reset for pricing band rates at the start of each
                billing period.

              - If FALSE, usage does not accumulate, and is reset for pricing bands at the
                start of each billing period.

          type: - **DEBIT**. Default setting. The amount calculated using the Pricing is added
                to the bill as a debit.

              - **PRODUCT_CREDIT**. The amount calculated using the Pricing is added to the
                bill as a credit _(negative amount)_. To prevent negative billing, the bill
                will be capped at the total of other line items for the same Product.

              - **GLOBAL_CREDIT**. The amount calculated using the Pricing is added to the
                bill as a credit _(negative amount)_. To prevent negative billing, the bill
                will be capped at the total of other line items for the entire bill, which
                might include other Products the Account consumes.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/organizations/{org_id}/pricings/{id}",
            body=maybe_transform(
                {
                    "pricing_bands": pricing_bands,
                    "start_date": start_date,
                    "accounting_product_id": accounting_product_id,
                    "aggregation_id": aggregation_id,
                    "code": code,
                    "compound_aggregation_id": compound_aggregation_id,
                    "cumulative": cumulative,
                    "description": description,
                    "end_date": end_date,
                    "minimum_spend": minimum_spend,
                    "minimum_spend_bill_in_advance": minimum_spend_bill_in_advance,
                    "minimum_spend_description": minimum_spend_description,
                    "overage_pricing_bands": overage_pricing_bands,
                    "plan_id": plan_id,
                    "plan_template_id": plan_template_id,
                    "segment": segment,
                    "tiers_span_plan": tiers_span_plan,
                    "type": type,
                    "version": version,
                },
                pricing_update_params.PricingUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PricingResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        date: str | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        plan_id: str | NotGiven = NOT_GIVEN,
        plan_template_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursor[PricingResponse]:
        """
        Retrieve a list of Pricings filtered by date, Plan ID, PlanTemplate ID, or
        Pricing ID.

        Args:
          date: Date on which to retrieve active Pricings.

          ids: List of Pricing IDs to retrieve.

          next_token: `nextToken` for multi-page retrievals.

          page_size: Number of Pricings to retrieve per page.

          plan_id: UUID of the Plan to retrieve Pricings for.

          plan_template_id: UUID of the PlanTemplate to retrieve Pricings for.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/pricings",
            page=SyncCursor[PricingResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "date": date,
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                        "plan_id": plan_id,
                        "plan_template_id": plan_template_id,
                    },
                    pricing_list_params.PricingListParams,
                ),
            ),
            model=PricingResponse,
        )

    def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PricingResponse:
        """
        Delete the Pricing with the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/organizations/{org_id}/pricings/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PricingResponse,
        )


class AsyncPricingsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPricingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPricingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPricingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncPricingsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        org_id: str | None = None,
        pricing_bands: Iterable[PricingBand],
        start_date: Union[str, datetime],
        accounting_product_id: str | NotGiven = NOT_GIVEN,
        aggregation_id: str | NotGiven = NOT_GIVEN,
        code: str | NotGiven = NOT_GIVEN,
        compound_aggregation_id: str | NotGiven = NOT_GIVEN,
        cumulative: bool | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        end_date: Union[str, datetime] | NotGiven = NOT_GIVEN,
        minimum_spend: float | NotGiven = NOT_GIVEN,
        minimum_spend_bill_in_advance: bool | NotGiven = NOT_GIVEN,
        minimum_spend_description: str | NotGiven = NOT_GIVEN,
        overage_pricing_bands: Iterable[PricingBand] | NotGiven = NOT_GIVEN,
        plan_id: str | NotGiven = NOT_GIVEN,
        plan_template_id: str | NotGiven = NOT_GIVEN,
        segment: Dict[str, str] | NotGiven = NOT_GIVEN,
        tiers_span_plan: bool | NotGiven = NOT_GIVEN,
        type: Literal["DEBIT", "PRODUCT_CREDIT", "GLOBAL_CREDIT"] | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PricingResponse:
        """
        Create a new Pricing.

        **Note:** Either `planId` or `planTemplateId` request parameters are required
        for this call to be valid. If you omit both, then you will receive a validation
        error.

        Args:
          pricing_bands

          start_date: The start date _(in ISO-8601 format)_ for when the Pricing starts to be active
              for the Plan of Plan Template._(Required)_

          accounting_product_id: Optional Product ID this Pricing should be attributed to for accounting purposes

          aggregation_id: UUID of the Aggregation used to create the Pricing. Use this when creating a
              Pricing for a segmented aggregation.

          code: Unique short code for the Pricing.

          compound_aggregation_id: UUID of the Compound Aggregation used to create the Pricing.

          cumulative: Controls whether or not charge rates under a set of pricing bands configured for
              a Pricing are applied according to each separate band or at the highest band
              reached.

              _(Optional)_. The default value is **FALSE**.

              - When TRUE, at billing charge rates are applied according to each separate
                band.

              - When FALSE, at billing charge rates are applied according to highest band
                reached.

              **NOTE:** Use the `cumulative` parameter to create the type of Pricing you
              require. For example, for Tiered Pricing set to **TRUE**; for Volume Pricing,
              set to **FALSE**.

          description: Displayed on Bill line items.

          end_date: The end date _(in ISO-8601 format)_ for when the Pricing ceases to be active for
              the Plan or Plan Template.

              _(Optional)_ If not specified, the Pricing remains active indefinitely.

          minimum_spend: The minimum spend amount per billing cycle for end customer Accounts on a Plan
              to which the Pricing is applied.

          minimum_spend_bill_in_advance: The default value is **FALSE**.

              - When TRUE, minimum spend is billed at the start of each billing period.

              - When FALSE, minimum spend is billed at the end of each billing period.

              _(Optional)_. Overrides the setting at Organization level for minimum spend
              billing in arrears/in advance.

          minimum_spend_description: Minimum spend description _(displayed on the bill line item)_.

          overage_pricing_bands: Specify Prepayment/Balance overage pricing in pricing bands for the case of a
              **Tiered** pricing structure.

          plan_id: UUID of the Plan the Pricing is created for.

          plan_template_id: UUID of the Plan Template the Pricing is created for.

          segment: Specifies the segment value which you are defining a Pricing for using this
              call:

              - For each segment value defined on a Segmented Aggregation you must create a
                separate Pricing and use the appropriate `aggregationId` parameter for the
                call.
              - If you specify a segment value that has not been defined for the Aggregation,
                you'll receive an error.
              - If you've defined segment values for the Aggregation using a single wildcard
                or multiple wildcards, you can create Pricing for these wildcard segment
                values also.

              For more details on creating Pricings for segment values on a Segmented
              Aggregation using this call, together with some examples, see the
              [Using API Call to Create Segmented Pricings](https://www.m3ter.com/docs/guides/plans-and-pricing/pricing-plans/pricing-plans-using-segmented-aggregations#using-api-call-to-create-a-segmented-pricing)
              in our User Documentation.

          tiers_span_plan: The default value is **FALSE**.

              - If TRUE, usage accumulates over the entire period the priced Plan is active
                for the account, and is not reset for pricing band rates at the start of each
                billing period.

              - If FALSE, usage does not accumulate, and is reset for pricing bands at the
                start of each billing period.

          type: - **DEBIT**. Default setting. The amount calculated using the Pricing is added
                to the bill as a debit.

              - **PRODUCT_CREDIT**. The amount calculated using the Pricing is added to the
                bill as a credit _(negative amount)_. To prevent negative billing, the bill
                will be capped at the total of other line items for the same Product.

              - **GLOBAL_CREDIT**. The amount calculated using the Pricing is added to the
                bill as a credit _(negative amount)_. To prevent negative billing, the bill
                will be capped at the total of other line items for the entire bill, which
                might include other Products the Account consumes.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/pricings",
            body=await async_maybe_transform(
                {
                    "pricing_bands": pricing_bands,
                    "start_date": start_date,
                    "accounting_product_id": accounting_product_id,
                    "aggregation_id": aggregation_id,
                    "code": code,
                    "compound_aggregation_id": compound_aggregation_id,
                    "cumulative": cumulative,
                    "description": description,
                    "end_date": end_date,
                    "minimum_spend": minimum_spend,
                    "minimum_spend_bill_in_advance": minimum_spend_bill_in_advance,
                    "minimum_spend_description": minimum_spend_description,
                    "overage_pricing_bands": overage_pricing_bands,
                    "plan_id": plan_id,
                    "plan_template_id": plan_template_id,
                    "segment": segment,
                    "tiers_span_plan": tiers_span_plan,
                    "type": type,
                    "version": version,
                },
                pricing_create_params.PricingCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PricingResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PricingResponse:
        """
        Retrieve the Pricing with the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/pricings/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PricingResponse,
        )

    async def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        pricing_bands: Iterable[PricingBand],
        start_date: Union[str, datetime],
        accounting_product_id: str | NotGiven = NOT_GIVEN,
        aggregation_id: str | NotGiven = NOT_GIVEN,
        code: str | NotGiven = NOT_GIVEN,
        compound_aggregation_id: str | NotGiven = NOT_GIVEN,
        cumulative: bool | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        end_date: Union[str, datetime] | NotGiven = NOT_GIVEN,
        minimum_spend: float | NotGiven = NOT_GIVEN,
        minimum_spend_bill_in_advance: bool | NotGiven = NOT_GIVEN,
        minimum_spend_description: str | NotGiven = NOT_GIVEN,
        overage_pricing_bands: Iterable[PricingBand] | NotGiven = NOT_GIVEN,
        plan_id: str | NotGiven = NOT_GIVEN,
        plan_template_id: str | NotGiven = NOT_GIVEN,
        segment: Dict[str, str] | NotGiven = NOT_GIVEN,
        tiers_span_plan: bool | NotGiven = NOT_GIVEN,
        type: Literal["DEBIT", "PRODUCT_CREDIT", "GLOBAL_CREDIT"] | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PricingResponse:
        """
        Update Pricing for the given UUID.

        **Note:** Either `planId` or `planTemplateId` request parameters are required
        for this call to be valid. If you omit both, then you will receive a validation
        error.

        Args:
          pricing_bands

          start_date: The start date _(in ISO-8601 format)_ for when the Pricing starts to be active
              for the Plan of Plan Template._(Required)_

          accounting_product_id: Optional Product ID this Pricing should be attributed to for accounting purposes

          aggregation_id: UUID of the Aggregation used to create the Pricing. Use this when creating a
              Pricing for a segmented aggregation.

          code: Unique short code for the Pricing.

          compound_aggregation_id: UUID of the Compound Aggregation used to create the Pricing.

          cumulative: Controls whether or not charge rates under a set of pricing bands configured for
              a Pricing are applied according to each separate band or at the highest band
              reached.

              _(Optional)_. The default value is **FALSE**.

              - When TRUE, at billing charge rates are applied according to each separate
                band.

              - When FALSE, at billing charge rates are applied according to highest band
                reached.

              **NOTE:** Use the `cumulative` parameter to create the type of Pricing you
              require. For example, for Tiered Pricing set to **TRUE**; for Volume Pricing,
              set to **FALSE**.

          description: Displayed on Bill line items.

          end_date: The end date _(in ISO-8601 format)_ for when the Pricing ceases to be active for
              the Plan or Plan Template.

              _(Optional)_ If not specified, the Pricing remains active indefinitely.

          minimum_spend: The minimum spend amount per billing cycle for end customer Accounts on a Plan
              to which the Pricing is applied.

          minimum_spend_bill_in_advance: The default value is **FALSE**.

              - When TRUE, minimum spend is billed at the start of each billing period.

              - When FALSE, minimum spend is billed at the end of each billing period.

              _(Optional)_. Overrides the setting at Organization level for minimum spend
              billing in arrears/in advance.

          minimum_spend_description: Minimum spend description _(displayed on the bill line item)_.

          overage_pricing_bands: Specify Prepayment/Balance overage pricing in pricing bands for the case of a
              **Tiered** pricing structure.

          plan_id: UUID of the Plan the Pricing is created for.

          plan_template_id: UUID of the Plan Template the Pricing is created for.

          segment: Specifies the segment value which you are defining a Pricing for using this
              call:

              - For each segment value defined on a Segmented Aggregation you must create a
                separate Pricing and use the appropriate `aggregationId` parameter for the
                call.
              - If you specify a segment value that has not been defined for the Aggregation,
                you'll receive an error.
              - If you've defined segment values for the Aggregation using a single wildcard
                or multiple wildcards, you can create Pricing for these wildcard segment
                values also.

              For more details on creating Pricings for segment values on a Segmented
              Aggregation using this call, together with some examples, see the
              [Using API Call to Create Segmented Pricings](https://www.m3ter.com/docs/guides/plans-and-pricing/pricing-plans/pricing-plans-using-segmented-aggregations#using-api-call-to-create-a-segmented-pricing)
              in our User Documentation.

          tiers_span_plan: The default value is **FALSE**.

              - If TRUE, usage accumulates over the entire period the priced Plan is active
                for the account, and is not reset for pricing band rates at the start of each
                billing period.

              - If FALSE, usage does not accumulate, and is reset for pricing bands at the
                start of each billing period.

          type: - **DEBIT**. Default setting. The amount calculated using the Pricing is added
                to the bill as a debit.

              - **PRODUCT_CREDIT**. The amount calculated using the Pricing is added to the
                bill as a credit _(negative amount)_. To prevent negative billing, the bill
                will be capped at the total of other line items for the same Product.

              - **GLOBAL_CREDIT**. The amount calculated using the Pricing is added to the
                bill as a credit _(negative amount)_. To prevent negative billing, the bill
                will be capped at the total of other line items for the entire bill, which
                might include other Products the Account consumes.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/organizations/{org_id}/pricings/{id}",
            body=await async_maybe_transform(
                {
                    "pricing_bands": pricing_bands,
                    "start_date": start_date,
                    "accounting_product_id": accounting_product_id,
                    "aggregation_id": aggregation_id,
                    "code": code,
                    "compound_aggregation_id": compound_aggregation_id,
                    "cumulative": cumulative,
                    "description": description,
                    "end_date": end_date,
                    "minimum_spend": minimum_spend,
                    "minimum_spend_bill_in_advance": minimum_spend_bill_in_advance,
                    "minimum_spend_description": minimum_spend_description,
                    "overage_pricing_bands": overage_pricing_bands,
                    "plan_id": plan_id,
                    "plan_template_id": plan_template_id,
                    "segment": segment,
                    "tiers_span_plan": tiers_span_plan,
                    "type": type,
                    "version": version,
                },
                pricing_update_params.PricingUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PricingResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        date: str | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        plan_id: str | NotGiven = NOT_GIVEN,
        plan_template_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[PricingResponse, AsyncCursor[PricingResponse]]:
        """
        Retrieve a list of Pricings filtered by date, Plan ID, PlanTemplate ID, or
        Pricing ID.

        Args:
          date: Date on which to retrieve active Pricings.

          ids: List of Pricing IDs to retrieve.

          next_token: `nextToken` for multi-page retrievals.

          page_size: Number of Pricings to retrieve per page.

          plan_id: UUID of the Plan to retrieve Pricings for.

          plan_template_id: UUID of the PlanTemplate to retrieve Pricings for.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/pricings",
            page=AsyncCursor[PricingResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "date": date,
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                        "plan_id": plan_id,
                        "plan_template_id": plan_template_id,
                    },
                    pricing_list_params.PricingListParams,
                ),
            ),
            model=PricingResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PricingResponse:
        """
        Delete the Pricing with the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/organizations/{org_id}/pricings/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PricingResponse,
        )


class PricingsResourceWithRawResponse:
    def __init__(self, pricings: PricingsResource) -> None:
        self._pricings = pricings

        self.create = to_raw_response_wrapper(
            pricings.create,
        )
        self.retrieve = to_raw_response_wrapper(
            pricings.retrieve,
        )
        self.update = to_raw_response_wrapper(
            pricings.update,
        )
        self.list = to_raw_response_wrapper(
            pricings.list,
        )
        self.delete = to_raw_response_wrapper(
            pricings.delete,
        )


class AsyncPricingsResourceWithRawResponse:
    def __init__(self, pricings: AsyncPricingsResource) -> None:
        self._pricings = pricings

        self.create = async_to_raw_response_wrapper(
            pricings.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            pricings.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            pricings.update,
        )
        self.list = async_to_raw_response_wrapper(
            pricings.list,
        )
        self.delete = async_to_raw_response_wrapper(
            pricings.delete,
        )


class PricingsResourceWithStreamingResponse:
    def __init__(self, pricings: PricingsResource) -> None:
        self._pricings = pricings

        self.create = to_streamed_response_wrapper(
            pricings.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            pricings.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            pricings.update,
        )
        self.list = to_streamed_response_wrapper(
            pricings.list,
        )
        self.delete = to_streamed_response_wrapper(
            pricings.delete,
        )


class AsyncPricingsResourceWithStreamingResponse:
    def __init__(self, pricings: AsyncPricingsResource) -> None:
        self._pricings = pricings

        self.create = async_to_streamed_response_wrapper(
            pricings.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            pricings.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            pricings.update,
        )
        self.list = async_to_streamed_response_wrapper(
            pricings.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            pricings.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["CounterAdjustmentCreateParams"]


class CounterAdjustmentCreateParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    account_id: Required[Annotated[str, PropertyInfo(alias="accountId")]]
    """The Account ID the CounterAdjustment is created for."""

    counter_id: Required[Annotated[str, PropertyInfo(alias="counterId")]]
    """The ID of the Counter used for the CounterAdjustment on the Account."""

    date: Required[str]
    """
    The date the CounterAdjustment is created for the Account _(in ISO-8601 date
    format)_.

    **Note:** CounterAdjustments on Accounts are supported down to a _specific day_
    of granularity - you cannot create more than one CounterAdjustment for any given
    day using the same Counter and you'll receive an error if you try to do this.
    """

    value: Required[int]
    """Integer Value of the Counter used for the CounterAdjustment.

    **Note:** Use the new absolute value for the Counter for the selected date - if
    it was 15 and has increased to 20, enter 20; if it was 15 and has decreased to
    10, enter 10. _Do not enter_ the plus or minus value relative to the previous
    Counter value on the Account.
    """

    purchase_order_number: Annotated[str, PropertyInfo(alias="purchaseOrderNumber")]
    """Purchase Order Number for the Counter Adjustment. _(Optional)_"""

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .usage import (
    UsageResource,
    AsyncUsageResource,
    UsageResourceWithRawResponse,
    AsyncUsageResourceWithRawResponse,
    UsageResourceWithStreamingResponse,
    AsyncUsageResourceWithStreamingResponse,
)
from .file_uploads import (
    FileUploadsResource,
    AsyncFileUploadsResource,
    FileUploadsResourceWithRawResponse,
    AsyncFileUploadsResourceWithRawResponse,
    FileUploadsResourceWithStreamingResponse,
    AsyncFileUploadsResourceWithStreamingResponse,
)

__all__ = [
    "FileUploadsResource",
    "AsyncFileUploadsResource",
    "FileUploadsResourceWithRawResponse",
    "AsyncFileUploadsResourceWithRawResponse",
    "FileUploadsResourceWithStreamingResponse",
    "AsyncFileUploadsResourceWithStreamingResponse",
    "UsageResource",
    "AsyncUsageResource",
    "UsageResourceWithRawResponse",
    "AsyncUsageResourceWithRawResponse",
    "UsageResourceWithStreamingResponse",
    "AsyncUsageResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["BillJobListParams"]


class BillJobListParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    active: str
    """
    Boolean filter to retrieve only active BillJobs and exclude completed or
    cancelled BillJobs from the results.

    - TRUE - only active BillJobs.
    - FALSE - all BillJobs including completed and cancelled BillJobs.
    """

    next_token: Annotated[str, PropertyInfo(alias="nextToken")]
    """The `nextToken` for multi-page retrievals.

    It is used to fetch the next page of BillJobs in a paginated list.
    """

    page_size: Annotated[int, PropertyInfo(alias="pageSize")]
    """Specifies the maximum number of BillJobs to retrieve per page."""

    status: str
    """Filter BillJobs by specific status.

    Allows for targeted retrieval of BillJobs based on their current processing
    status.

    Possible states are:

    - PENDING
    - INITIALIZING
    - RUNNING
    - COMPLETE
    - CANCELLED
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from datetime import datetime

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursor, AsyncCursor
from ..._base_client import AsyncPaginator, make_request_options
from ...types.balances import transaction_list_params, transaction_create_params
from ...types.balances.transaction_response import TransactionResponse
from ...types.balances.transaction_summary_response import TransactionSummaryResponse

__all__ = ["TransactionsResource", "AsyncTransactionsResource"]


class TransactionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TransactionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return TransactionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TransactionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return TransactionsResourceWithStreamingResponse(self)

    def create(
        self,
        balance_id: str,
        *,
        org_id: str | None = None,
        amount: float,
        applied_date: Union[str, datetime] | NotGiven = NOT_GIVEN,
        currency_paid: str | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        paid: float | NotGiven = NOT_GIVEN,
        transaction_date: Union[str, datetime] | NotGiven = NOT_GIVEN,
        transaction_type_id: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TransactionResponse:
        """Add a Transaction to a Balance.

        This endpoint allows you to create a new
        Transaction amount for a Balance. This amount then becomes available at billing
        for draw-down to cover charges due. The Transaction details should be provided
        in the request body.

        Before you can add a Transaction amount, you must first set up Transaction Types
        at the Organization Level - see the
        [Transaction Type](https://www.m3ter.com/docs/api#tag/TransactionType) section
        in this API Reference for more details. You can then use this call to add an
        instance of a Transaction Type to a Balance.

        **Note:** If you have a customer whose payment is in a different currency to the
        Balance currency, you can use the `paid` and `paidCurrency` request parameters
        to record the amount paid and alternative currency respectively. For example,
        you might add a Transaction amount of 200 USD to a Balance on a customer Account
        where the customer actually paid you 50 units in virtual currency X.

        Args:
          amount: The financial value of the transaction.

          applied_date: The date _(in ISO 8601 format)_ when the Balance transaction was applied.

          currency_paid: The currency code of the payment if it differs from the Balance currency. For
              example: USD, GBP or EUR.

          description: A brief description explaining the purpose and context of the transaction.

          paid: The payment amount if the payment currency differs from the Balance currency.

          transaction_date: The date _(in ISO 8601 format)_ when the transaction occurred.

          transaction_type_id: The unique identifier (UUID) of the transaction type. This is obtained from the
              list of created Transaction Types within the Organization Configuration.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not balance_id:
            raise ValueError(f"Expected a non-empty value for `balance_id` but received {balance_id!r}")
        return self._post(
            f"/organizations/{org_id}/balances/{balance_id}/transactions",
            body=maybe_transform(
                {
                    "amount": amount,
                    "applied_date": applied_date,
                    "currency_paid": currency_paid,
                    "description": description,
                    "paid": paid,
                    "transaction_date": transaction_date,
                    "transaction_type_id": transaction_type_id,
                    "version": version,
                },
                transaction_create_params.TransactionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TransactionResponse,
        )

    def list(
        self,
        balance_id: str,
        *,
        org_id: str | None = None,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        transaction_type_id: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursor[TransactionResponse]:
        """
        Retrieve all Transactions for a specific Balance.

        This endpoint returns a list of all Transactions associated with a specific
        Balance. You can paginate through the Transactions by using the `pageSize` and
        `nextToken` parameters.

        Args:
          next_token: `nextToken` for multi page retrievals. A token for retrieving the next page of
              transactions. You'll get this from the response to your request.

          page_size: The maximum number of transactions to return per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not balance_id:
            raise ValueError(f"Expected a non-empty value for `balance_id` but received {balance_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/balances/{balance_id}/transactions",
            page=SyncCursor[TransactionResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "next_token": next_token,
                        "page_size": page_size,
                        "transaction_type_id": transaction_type_id,
                    },
                    transaction_list_params.TransactionListParams,
                ),
            ),
            model=TransactionResponse,
        )

    def summary(
        self,
        balance_id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TransactionSummaryResponse:
        """
        Retrieves the Balance Transactions Summary for a given Balance.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not balance_id:
            raise ValueError(f"Expected a non-empty value for `balance_id` but received {balance_id!r}")
        return self._get(
            f"/organizations/{org_id}/balances/{balance_id}/transactions/summary",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TransactionSummaryResponse,
        )


class AsyncTransactionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTransactionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTransactionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTransactionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncTransactionsResourceWithStreamingResponse(self)

    async def create(
        self,
        balance_id: str,
        *,
        org_id: str | None = None,
        amount: float,
        applied_date: Union[str, datetime] | NotGiven = NOT_GIVEN,
        currency_paid: str | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        paid: float | NotGiven = NOT_GIVEN,
        transaction_date: Union[str, datetime] | NotGiven = NOT_GIVEN,
        transaction_type_id: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TransactionResponse:
        """Add a Transaction to a Balance.

        This endpoint allows you to create a new
        Transaction amount for a Balance. This amount then becomes available at billing
        for draw-down to cover charges due. The Transaction details should be provided
        in the request body.

        Before you can add a Transaction amount, you must first set up Transaction Types
        at the Organization Level - see the
        [Transaction Type](https://www.m3ter.com/docs/api#tag/TransactionType) section
        in this API Reference for more details. You can then use this call to add an
        instance of a Transaction Type to a Balance.

        **Note:** If you have a customer whose payment is in a different currency to the
        Balance currency, you can use the `paid` and `paidCurrency` request parameters
        to record the amount paid and alternative currency respectively. For example,
        you might add a Transaction amount of 200 USD to a Balance on a customer Account
        where the customer actually paid you 50 units in virtual currency X.

        Args:
          amount: The financial value of the transaction.

          applied_date: The date _(in ISO 8601 format)_ when the Balance transaction was applied.

          currency_paid: The currency code of the payment if it differs from the Balance currency. For
              example: USD, GBP or EUR.

          description: A brief description explaining the purpose and context of the transaction.

          paid: The payment amount if the payment currency differs from the Balance currency.

          transaction_date: The date _(in ISO 8601 format)_ when the transaction occurred.

          transaction_type_id: The unique identifier (UUID) of the transaction type. This is obtained from the
              list of created Transaction Types within the Organization Configuration.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not balance_id:
            raise ValueError(f"Expected a non-empty value for `balance_id` but received {balance_id!r}")
        return await self._post(
            f"/organizations/{org_id}/balances/{balance_id}/transactions",
            body=await async_maybe_transform(
                {
                    "amount": amount,
                    "applied_date": applied_date,
                    "currency_paid": currency_paid,
                    "description": description,
                    "paid": paid,
                    "transaction_date": transaction_date,
                    "transaction_type_id": transaction_type_id,
                    "version": version,
                },
                transaction_create_params.TransactionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TransactionResponse,
        )

    def list(
        self,
        balance_id: str,
        *,
        org_id: str | None = None,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        transaction_type_id: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[TransactionResponse, AsyncCursor[TransactionResponse]]:
        """
        Retrieve all Transactions for a specific Balance.

        This endpoint returns a list of all Transactions associated with a specific
        Balance. You can paginate through the Transactions by using the `pageSize` and
        `nextToken` parameters.

        Args:
          next_token: `nextToken` for multi page retrievals. A token for retrieving the next page of
              transactions. You'll get this from the response to your request.

          page_size: The maximum number of transactions to return per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not balance_id:
            raise ValueError(f"Expected a non-empty value for `balance_id` but received {balance_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/balances/{balance_id}/transactions",
            page=AsyncCursor[TransactionResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "next_token": next_token,
                        "page_size": page_size,
                        "transaction_type_id": transaction_type_id,
                    },
                    transaction_list_params.TransactionListParams,
                ),
            ),
            model=TransactionResponse,
        )

    async def summary(
        self,
        balance_id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TransactionSummaryResponse:
        """
        Retrieves the Balance Transactions Summary for a given Balance.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not balance_id:
            raise ValueError(f"Expected a non-empty value for `balance_id` but received {balance_id!r}")
        return await self._get(
            f"/organizations/{org_id}/balances/{balance_id}/transactions/summary",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TransactionSummaryResponse,
        )


class TransactionsResourceWithRawResponse:
    def __init__(self, transactions: TransactionsResource) -> None:
        self._transactions = transactions

        self.create = to_raw_response_wrapper(
            transactions.create,
        )
        self.list = to_raw_response_wrapper(
            transactions.list,
        )
        self.summary = to_raw_response_wrapper(
            transactions.summary,
        )


class AsyncTransactionsResourceWithRawResponse:
    def __init__(self, transactions: AsyncTransactionsResource) -> None:
        self._transactions = transactions

        self.create = async_to_raw_response_wrapper(
            transactions.create,
        )
        self.list = async_to_raw_response_wrapper(
            transactions.list,
        )
        self.summary = async_to_raw_response_wrapper(
            transactions.summary,
        )


class TransactionsResourceWithStreamingResponse:
    def __init__(self, transactions: TransactionsResource) -> None:
        self._transactions = transactions

        self.create = to_streamed_response_wrapper(
            transactions.create,
        )
        self.list = to_streamed_response_wrapper(
            transactions.list,
        )
        self.summary = to_streamed_response_wrapper(
            transactions.summary,
        )


class AsyncTransactionsResourceWithStreamingResponse:
    def __init__(self, transactions: AsyncTransactionsResource) -> None:
        self._transactions = transactions

        self.create = async_to_streamed_response_wrapper(
            transactions.create,
        )
        self.list = async_to_streamed_response_wrapper(
            transactions.list,
        )
        self.summary = async_to_streamed_response_wrapper(
            transactions.summary,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .bills import (
    BillsResource,
    AsyncBillsResource,
    BillsResourceWithRawResponse,
    AsyncBillsResourceWithRawResponse,
    BillsResourceWithStreamingResponse,
    AsyncBillsResourceWithStreamingResponse,
)
from .line_items import (
    LineItemsResource,
    AsyncLineItemsResource,
    LineItemsResourceWithRawResponse,
    AsyncLineItemsResourceWithRawResponse,
    LineItemsResourceWithStreamingResponse,
    AsyncLineItemsResourceWithStreamingResponse,
)
from .debit_line_items import (
    DebitLineItemsResource,
    AsyncDebitLineItemsResource,
    DebitLineItemsResourceWithRawResponse,
    AsyncDebitLineItemsResourceWithRawResponse,
    DebitLineItemsResourceWithStreamingResponse,
    AsyncDebitLineItemsResourceWithStreamingResponse,
)
from .credit_line_items import (
    CreditLineItemsResource,
    AsyncCreditLineItemsResource,
    CreditLineItemsResourceWithRawResponse,
    AsyncCreditLineItemsResourceWithRawResponse,
    CreditLineItemsResourceWithStreamingResponse,
    AsyncCreditLineItemsResourceWithStreamingResponse,
)

__all__ = [
    "CreditLineItemsResource",
    "AsyncCreditLineItemsResource",
    "CreditLineItemsResourceWithRawResponse",
    "AsyncCreditLineItemsResourceWithRawResponse",
    "CreditLineItemsResourceWithStreamingResponse",
    "AsyncCreditLineItemsResourceWithStreamingResponse",
    "DebitLineItemsResource",
    "AsyncDebitLineItemsResource",
    "DebitLineItemsResourceWithRawResponse",
    "AsyncDebitLineItemsResourceWithRawResponse",
    "DebitLineItemsResourceWithStreamingResponse",
    "AsyncDebitLineItemsResourceWithStreamingResponse",
    "LineItemsResource",
    "AsyncLineItemsResource",
    "LineItemsResourceWithRawResponse",
    "AsyncLineItemsResourceWithRawResponse",
    "LineItemsResourceWithStreamingResponse",
    "AsyncLineItemsResourceWithStreamingResponse",
    "BillsResource",
    "AsyncBillsResource",
    "BillsResourceWithRawResponse",
    "AsyncBillsResourceWithRawResponse",
    "BillsResourceWithStreamingResponse",
    "AsyncBillsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union
from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["ProductUpdateParams"]


class ProductUpdateParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    code: Required[str]
    """A unique short code to identify the Product.

    It should not contain control chracters or spaces.
    """

    name: Required[str]
    """Descriptive name for the Product providing context and information."""

    custom_fields: Annotated[Dict[str, Union[str, float]], PropertyInfo(alias="customFields")]
    """User defined fields enabling you to attach custom data.

    The value for a custom field can be either a string or a number.

    If `customFields` can also be defined for this entity at the Organizational
    level, `customField` values defined at individual level override values of
    `customFields` with the same name defined at Organization level.

    See
    [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
    in the m3ter documentation for more information.
    """

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["CommitmentSearchParams"]


class CommitmentSearchParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    from_document: Annotated[int, PropertyInfo(alias="fromDocument")]
    """`fromDocument` for multi page retrievals."""

    operator: Literal["AND", "OR"]
    """Search Operator to be used while querying search."""

    page_size: Annotated[int, PropertyInfo(alias="pageSize")]
    """Number of Commitments to retrieve per page.

    **NOTE:** If not defined, default is 10.
    """

    search_query: Annotated[str, PropertyInfo(alias="searchQuery")]
    """Query for data using special syntax:

    - Query parameters should be delimited using $ (dollar sign).
    - Allowed comparators are:
      - (greater than) >
      - (greater than or equal to) >=
      - (equal to) :
      - (less than) <
      - (less than or equal to) <=
      - (match phrase/prefix) ~
    - Allowed parameters: startDate, endDate, contractId, accountId, productId,
      productIds, id, createdBy, dtCreated, lastModifiedBy, ids.
    - Query example:
      - searchQuery=startDate>2023-01-01$accountId:062085ab-a301-4f21-a081-411020864452.
      - This query is translated into: find commitments where the startDate is older
        than 2023-01-01 AND the accountId is equal to
        062085ab-a301-4f21-a081-411020864452.

    **Note:** Using the ~ match phrase/prefix comparator. For best results, we
    recommend treating this as a "starts with" comparator for your search query.
    """

    sort_by: Annotated[str, PropertyInfo(alias="sortBy")]
    """Name of the parameter on which sorting is performed.

    Use any field available on the Commitment entity to sort by, such as
    `accountId`, `endDate`, and so on.
    """

    sort_order: Annotated[Literal["ASC", "DESC"], PropertyInfo(alias="sortOrder")]
    """Sorting order."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, List, cast
from typing_extensions import Literal, overload

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    required_args,
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursor, AsyncCursor
from ..._base_client import AsyncPaginator, make_request_options
from ...types.data_exports import schedule_list_params, schedule_create_params, schedule_update_params
from ...types.data_exports.schedule_list_response import ScheduleListResponse
from ...types.data_exports.schedule_create_response import ScheduleCreateResponse
from ...types.data_exports.schedule_delete_response import ScheduleDeleteResponse
from ...types.data_exports.schedule_update_response import ScheduleUpdateResponse
from ...types.data_exports.schedule_retrieve_response import ScheduleRetrieveResponse

__all__ = ["SchedulesResource", "AsyncSchedulesResource"]


class SchedulesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SchedulesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return SchedulesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SchedulesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return SchedulesResourceWithStreamingResponse(self)

    @overload
    def create(
        self,
        *,
        org_id: str | None = None,
        operational_data_types: List[
            Literal[
                "BILLS",
                "COMMITMENTS",
                "ACCOUNTS",
                "BALANCES",
                "CONTRACTS",
                "ACCOUNT_PLANS",
                "AGGREGATIONS",
                "PLANS",
                "PRICING",
                "PRICING_BANDS",
                "BILL_LINE_ITEMS",
                "METERS",
                "PRODUCTS",
                "COMPOUND_AGGREGATIONS",
                "PLAN_GROUPS",
                "PLAN_GROUP_LINKS",
                "PLAN_TEMPLATES",
                "BALANCE_TRANSACTIONS",
            ]
        ],
        source_type: Literal["USAGE", "OPERATIONAL"],
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ScheduleCreateResponse:
        """Create a new Data Export Schedule.

        Each Schedule can be configured for exporting
        _only one_ of either Usage or Operational data:

        **Operational Data Exports**.

        - Use the `operationalDataTypes` parameter to specify the entities whose
          operational data you want to include in the export each time the Export
          Schedule runs.
        - For each of the entity types you select, each time the Export Schedule runs a
          separate file is compiled containing the operational data for all entities of
          that type that exist in your Organization

        **Usage Data Exports**.

        - Select the Meters and Accounts whose usage data you want to include in the
          export each time the Export Schedule runs.
        - If _don't want to aggregate_ the usage data collected by the selected Meters,
          use **ORIGINAL** for `aggregationFrequency`, which is the _default_. This
          means the raw usage data collected by any type of Data Fields and the values
          for any Derived Fields on the selected Meters will be included in the export.
        - If you _do want to aggregate_ the usage data collected by the selected Meters,
          use one of the other options for `aggregationFrequency`: **HOUR**, **DAY**,
          **WEEK**, or **MONTH**. You _must_ then also specified an `aggregation` method
          to be used on the usage data before export. Importantly, if you do aggregate
          usage data, only the usage data collected by any numeric Data Fields on the
          selected Meters - those of type **MEASURE**, **INCOME**, or **COST** - will be
          included in the export each time the Export Schedule runs.

        Args:
          operational_data_types: A list of the entities whose operational data is included in the data export.

          source_type

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        org_id: str | None = None,
        aggregation_frequency: Literal["ORIGINAL", "HOUR", "DAY", "WEEK", "MONTH"],
        source_type: Literal["USAGE", "OPERATIONAL"],
        time_period: Literal[
            "TODAY",
            "YESTERDAY",
            "WEEK_TO_DATE",
            "CURRENT_MONTH",
            "LAST_30_DAYS",
            "LAST_35_DAYS",
            "PREVIOUS_WEEK",
            "PREVIOUS_MONTH",
        ],
        account_ids: List[str] | NotGiven = NOT_GIVEN,
        aggregation: Literal["SUM", "MIN", "MAX", "COUNT", "LATEST", "MEAN"] | NotGiven = NOT_GIVEN,
        meter_ids: List[str] | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ScheduleCreateResponse:
        """Create a new Data Export Schedule.

        Each Schedule can be configured for exporting
        _only one_ of either Usage or Operational data:

        **Operational Data Exports**.

        - Use the `operationalDataTypes` parameter to specify the entities whose
          operational data you want to include in the export each time the Export
          Schedule runs.
        - For each of the entity types you select, each time the Export Schedule runs a
          separate file is compiled containing the operational data for all entities of
          that type that exist in your Organization

        **Usage Data Exports**.

        - Select the Meters and Accounts whose usage data you want to include in the
          export each time the Export Schedule runs.
        - If _don't want to aggregate_ the usage data collected by the selected Meters,
          use **ORIGINAL** for `aggregationFrequency`, which is the _default_. This
          means the raw usage data collected by any type of Data Fields and the values
          for any Derived Fields on the selected Meters will be included in the export.
        - If you _do want to aggregate_ the usage data collected by the selected Meters,
          use one of the other options for `aggregationFrequency`: **HOUR**, **DAY**,
          **WEEK**, or **MONTH**. You _must_ then also specified an `aggregation` method
          to be used on the usage data before export. Importantly, if you do aggregate
          usage data, only the usage data collected by any numeric Data Fields on the
          selected Meters - those of type **MEASURE**, **INCOME**, or **COST** - will be
          included in the export each time the Export Schedule runs.

        Args:
          aggregation_frequency: Specifies the time period for the aggregation of usage data included each time
              the Data Export Schedule runs:

              - **ORIGINAL**. Usage data is _not aggregated_. If you select to not aggregate,
                then raw usage data measurements collected by all Data Field types and any
                Derived Fields on the selected Meters are included in the export. This is the
                _Default_.

              If you want to aggregate usage data for the Export Schedule you must define an
              `aggregationFrequency`:

              - **HOUR**. Aggregated hourly.
              - **DAY**. Aggregated daily.
              - **WEEK**. Aggregated weekly.
              - **MONTH**. Aggregated monthly.

              - If you select to aggregate usage data for a Export Schedule, then only the
                aggregated usage data collected by numeric Data Fields of type **MEASURE**,
                **INCOME**, or **COST** on selected Meters are included in the export.

              **NOTE**: If you define an `aggregationFrequency` other than **ORIGINAL** and do
              not define an `aggregation` method, then you'll receive and error.

          source_type

          time_period: Define a time period to control the range of usage data you want the data export
              to contain when it runs:

              - **TODAY**. Data collected for the current day up until the time the export
                runs.
              - **YESTERDAY**. Data collected for the day before the export runs - that is,
                the 24 hour period from midnight to midnight of the day before.
              - **WEEK_TO_DATE**. Data collected for the period covering the current week to
                the date and time the export runs, and weeks run Monday to Monday.
              - **CURRENT_MONTH**. Data collected for the current month in which the export is
                ran up to and including the date and time the export runs.
              - **LAST_30_DAYS**. Data collected for the 30 days prior to the date the export
                is ran.
              - **LAST_35_DAYS**. Data collected for the 35 days prior to the date the export
                is ran.
              - **PREVIOUS_WEEK**. Data collected for the previous full week period, and weeks
                run Monday to Monday.
              - **PREVIOUS_MONTH**. Data collected for the previous full month period.

              For more details and examples, see the
              [Time Period](https://www.m3ter.com/docs/guides/data-exports/creating-export-schedules#time-period)
              section in our main User Documentation.

          account_ids: List of account IDs for which the usage data will be exported.

          aggregation: Specifies the aggregation method applied to usage data collected in the numeric
              Data Fields of Meters included for the Data Export Schedule - that is, Data
              Fields of type **MEASURE**, **INCOME**, or **COST**:

              - **SUM**. Adds the values.
              - **MIN**. Uses the minimum value.
              - **MAX**. Uses the maximum value.
              - **COUNT**. Counts the number of values.
              - **LATEST**. Uses the most recent value. Note: Based on the timestamp `ts`
                value of usage data measurement submissions. If using this method, please
                ensure _distinct_ `ts` values are used for usage data measurement submissions.

          meter_ids: List of meter IDs for which the usage data will be exported.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["operational_data_types", "source_type"], ["aggregation_frequency", "source_type", "time_period"])
    def create(
        self,
        *,
        org_id: str | None = None,
        operational_data_types: List[
            Literal[
                "BILLS",
                "COMMITMENTS",
                "ACCOUNTS",
                "BALANCES",
                "CONTRACTS",
                "ACCOUNT_PLANS",
                "AGGREGATIONS",
                "PLANS",
                "PRICING",
                "PRICING_BANDS",
                "BILL_LINE_ITEMS",
                "METERS",
                "PRODUCTS",
                "COMPOUND_AGGREGATIONS",
                "PLAN_GROUPS",
                "PLAN_GROUP_LINKS",
                "PLAN_TEMPLATES",
                "BALANCE_TRANSACTIONS",
            ]
        ]
        | NotGiven = NOT_GIVEN,
        source_type: Literal["USAGE", "OPERATIONAL"],
        version: int | NotGiven = NOT_GIVEN,
        aggregation_frequency: Literal["ORIGINAL", "HOUR", "DAY", "WEEK", "MONTH"] | NotGiven = NOT_GIVEN,
        time_period: Literal[
            "TODAY",
            "YESTERDAY",
            "WEEK_TO_DATE",
            "CURRENT_MONTH",
            "LAST_30_DAYS",
            "LAST_35_DAYS",
            "PREVIOUS_WEEK",
            "PREVIOUS_MONTH",
        ]
        | NotGiven = NOT_GIVEN,
        account_ids: List[str] | NotGiven = NOT_GIVEN,
        aggregation: Literal["SUM", "MIN", "MAX", "COUNT", "LATEST", "MEAN"] | NotGiven = NOT_GIVEN,
        meter_ids: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ScheduleCreateResponse:
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return cast(
            ScheduleCreateResponse,
            self._post(
                f"/organizations/{org_id}/dataexports/schedules",
                body=maybe_transform(
                    {
                        "operational_data_types": operational_data_types,
                        "source_type": source_type,
                        "version": version,
                        "aggregation_frequency": aggregation_frequency,
                        "time_period": time_period,
                        "account_ids": account_ids,
                        "aggregation": aggregation,
                        "meter_ids": meter_ids,
                    },
                    schedule_create_params.ScheduleCreateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, ScheduleCreateResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ScheduleRetrieveResponse:
        """Retrieve a Data Export Schedule for the given UUID.

        Each Schedule can be
        configured for exporting _only one_ of either Usage or Operational data.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return cast(
            ScheduleRetrieveResponse,
            self._get(
                f"/organizations/{org_id}/dataexports/schedules/{id}",
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, ScheduleRetrieveResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    @overload
    def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        operational_data_types: List[
            Literal[
                "BILLS",
                "COMMITMENTS",
                "ACCOUNTS",
                "BALANCES",
                "CONTRACTS",
                "ACCOUNT_PLANS",
                "AGGREGATIONS",
                "PLANS",
                "PRICING",
                "PRICING_BANDS",
                "BILL_LINE_ITEMS",
                "METERS",
                "PRODUCTS",
                "COMPOUND_AGGREGATIONS",
                "PLAN_GROUPS",
                "PLAN_GROUP_LINKS",
                "PLAN_TEMPLATES",
                "BALANCE_TRANSACTIONS",
            ]
        ],
        source_type: Literal["USAGE", "OPERATIONAL"],
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ScheduleUpdateResponse:
        """Update a Data Export Schedule for the given UUID.

        Each Schedule can be
        configured for exporting _only one_ of either Usage or Operational data:

        **Operational Data Exports**.

        - Use the `operationalDataTypes` parameter to specify the entities whose
          operational data you want to include in the export each time the Export
          Schedule runs.
        - For each of the entity types you select, each time the Export Schedule runs a
          separate file is compiled containing the operational data for all entities of
          that type that exist in your Organization

        **Usage Data Exports**.

        - Select the Meters and Accounts whose usage data you want to include in the
          export each time the Export Schedule runs.
        - If _don't want to aggregate_ the usage data collected by the selected Meters,
          use **ORIGINAL** for `aggregationFrequency`, which is the _default_. This
          means the raw usage data collected by any type of Data Fields and the values
          for any Derived Fields on the selected Meters will be included in the export.
        - If you _do want to aggregate_ the usage data collected by the selected Meters,
          use one of the other options for `aggregationFrequency`: **HOUR**, **DAY**,
          **WEEK**, or **MONTH**. You _must_ then also specified an `aggregation` method
          to be used on the usage data before export. Importantly, if you do aggregate
          usage data, only the usage data collected by any numeric Data Fields on the
          selected Meters - those of type **MEASURE**, **INCOME**, or **COST** - will be
          included in the export each time the Export Schedule runs.

        Args:
          operational_data_types: A list of the entities whose operational data is included in the data export.

          source_type

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        aggregation_frequency: Literal["ORIGINAL", "HOUR", "DAY", "WEEK", "MONTH"],
        source_type: Literal["USAGE", "OPERATIONAL"],
        time_period: Literal[
            "TODAY",
            "YESTERDAY",
            "WEEK_TO_DATE",
            "CURRENT_MONTH",
            "LAST_30_DAYS",
            "LAST_35_DAYS",
            "PREVIOUS_WEEK",
            "PREVIOUS_MONTH",
        ],
        account_ids: List[str] | NotGiven = NOT_GIVEN,
        aggregation: Literal["SUM", "MIN", "MAX", "COUNT", "LATEST", "MEAN"] | NotGiven = NOT_GIVEN,
        meter_ids: List[str] | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ScheduleUpdateResponse:
        """Update a Data Export Schedule for the given UUID.

        Each Schedule can be
        configured for exporting _only one_ of either Usage or Operational data:

        **Operational Data Exports**.

        - Use the `operationalDataTypes` parameter to specify the entities whose
          operational data you want to include in the export each time the Export
          Schedule runs.
        - For each of the entity types you select, each time the Export Schedule runs a
          separate file is compiled containing the operational data for all entities of
          that type that exist in your Organization

        **Usage Data Exports**.

        - Select the Meters and Accounts whose usage data you want to include in the
          export each time the Export Schedule runs.
        - If _don't want to aggregate_ the usage data collected by the selected Meters,
          use **ORIGINAL** for `aggregationFrequency`, which is the _default_. This
          means the raw usage data collected by any type of Data Fields and the values
          for any Derived Fields on the selected Meters will be included in the export.
        - If you _do want to aggregate_ the usage data collected by the selected Meters,
          use one of the other options for `aggregationFrequency`: **HOUR**, **DAY**,
          **WEEK**, or **MONTH**. You _must_ then also specified an `aggregation` method
          to be used on the usage data before export. Importantly, if you do aggregate
          usage data, only the usage data collected by any numeric Data Fields on the
          selected Meters - those of type **MEASURE**, **INCOME**, or **COST** - will be
          included in the export each time the Export Schedule runs.

        Args:
          aggregation_frequency: Specifies the time period for the aggregation of usage data included each time
              the Data Export Schedule runs:

              - **ORIGINAL**. Usage data is _not aggregated_. If you select to not aggregate,
                then raw usage data measurements collected by all Data Field types and any
                Derived Fields on the selected Meters are included in the export. This is the
                _Default_.

              If you want to aggregate usage data for the Export Schedule you must define an
              `aggregationFrequency`:

              - **HOUR**. Aggregated hourly.
              - **DAY**. Aggregated daily.
              - **WEEK**. Aggregated weekly.
              - **MONTH**. Aggregated monthly.

              - If you select to aggregate usage data for a Export Schedule, then only the
                aggregated usage data collected by numeric Data Fields of type **MEASURE**,
                **INCOME**, or **COST** on selected Meters are included in the export.

              **NOTE**: If you define an `aggregationFrequency` other than **ORIGINAL** and do
              not define an `aggregation` method, then you'll receive and error.

          source_type

          time_period: Define a time period to control the range of usage data you want the data export
              to contain when it runs:

              - **TODAY**. Data collected for the current day up until the time the export
                runs.
              - **YESTERDAY**. Data collected for the day before the export runs - that is,
                the 24 hour period from midnight to midnight of the day before.
              - **WEEK_TO_DATE**. Data collected for the period covering the current week to
                the date and time the export runs, and weeks run Monday to Monday.
              - **CURRENT_MONTH**. Data collected for the current month in which the export is
                ran up to and including the date and time the export runs.
              - **LAST_30_DAYS**. Data collected for the 30 days prior to the date the export
                is ran.
              - **LAST_35_DAYS**. Data collected for the 35 days prior to the date the export
                is ran.
              - **PREVIOUS_WEEK**. Data collected for the previous full week period, and weeks
                run Monday to Monday.
              - **PREVIOUS_MONTH**. Data collected for the previous full month period.

              For more details and examples, see the
              [Time Period](https://www.m3ter.com/docs/guides/data-exports/creating-export-schedules#time-period)
              section in our main User Documentation.

          account_ids: List of account IDs for which the usage data will be exported.

          aggregation: Specifies the aggregation method applied to usage data collected in the numeric
              Data Fields of Meters included for the Data Export Schedule - that is, Data
              Fields of type **MEASURE**, **INCOME**, or **COST**:

              - **SUM**. Adds the values.
              - **MIN**. Uses the minimum value.
              - **MAX**. Uses the maximum value.
              - **COUNT**. Counts the number of values.
              - **LATEST**. Uses the most recent value. Note: Based on the timestamp `ts`
                value of usage data measurement submissions. If using this method, please
                ensure _distinct_ `ts` values are used for usage data measurement submissions.

          meter_ids: List of meter IDs for which the usage data will be exported.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["operational_data_types", "source_type"], ["aggregation_frequency", "source_type", "time_period"])
    def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        operational_data_types: List[
            Literal[
                "BILLS",
                "COMMITMENTS",
                "ACCOUNTS",
                "BALANCES",
                "CONTRACTS",
                "ACCOUNT_PLANS",
                "AGGREGATIONS",
                "PLANS",
                "PRICING",
                "PRICING_BANDS",
                "BILL_LINE_ITEMS",
                "METERS",
                "PRODUCTS",
                "COMPOUND_AGGREGATIONS",
                "PLAN_GROUPS",
                "PLAN_GROUP_LINKS",
                "PLAN_TEMPLATES",
                "BALANCE_TRANSACTIONS",
            ]
        ]
        | NotGiven = NOT_GIVEN,
        source_type: Literal["USAGE", "OPERATIONAL"],
        version: int | NotGiven = NOT_GIVEN,
        aggregation_frequency: Literal["ORIGINAL", "HOUR", "DAY", "WEEK", "MONTH"] | NotGiven = NOT_GIVEN,
        time_period: Literal[
            "TODAY",
            "YESTERDAY",
            "WEEK_TO_DATE",
            "CURRENT_MONTH",
            "LAST_30_DAYS",
            "LAST_35_DAYS",
            "PREVIOUS_WEEK",
            "PREVIOUS_MONTH",
        ]
        | NotGiven = NOT_GIVEN,
        account_ids: List[str] | NotGiven = NOT_GIVEN,
        aggregation: Literal["SUM", "MIN", "MAX", "COUNT", "LATEST", "MEAN"] | NotGiven = NOT_GIVEN,
        meter_ids: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ScheduleUpdateResponse:
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return cast(
            ScheduleUpdateResponse,
            self._put(
                f"/organizations/{org_id}/dataexports/schedules/{id}",
                body=maybe_transform(
                    {
                        "operational_data_types": operational_data_types,
                        "source_type": source_type,
                        "version": version,
                        "aggregation_frequency": aggregation_frequency,
                        "time_period": time_period,
                        "account_ids": account_ids,
                        "aggregation": aggregation,
                        "meter_ids": meter_ids,
                    },
                    schedule_update_params.ScheduleUpdateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, ScheduleUpdateResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        ids: List[str] | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursor[ScheduleListResponse]:
        """Retrieve a list of Data Export Schedules created for your Organization.

        You can
        filter the response by Schedules `ids`.

        The response will contain an array for both the operational and usage Data
        Export Schedules in your Organization.

        Args:
          ids: Data Export Schedule IDs to filter the returned list by.

          next_token: `nextToken` for multi page retrievals

          page_size: Number of schedules to retrieve per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/dataexports/schedules",
            page=SyncCursor[ScheduleListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    schedule_list_params.ScheduleListParams,
                ),
            ),
            model=ScheduleListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ScheduleDeleteResponse:
        """Delete the Data Export Schedule for the given UUID.

        Each Schedule can be
        configured for exporting _only one_ of either Usage or Operational data.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return cast(
            ScheduleDeleteResponse,
            self._delete(
                f"/organizations/{org_id}/dataexports/schedules/{id}",
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, ScheduleDeleteResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class AsyncSchedulesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSchedulesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSchedulesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSchedulesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncSchedulesResourceWithStreamingResponse(self)

    @overload
    async def create(
        self,
        *,
        org_id: str | None = None,
        operational_data_types: List[
            Literal[
                "BILLS",
                "COMMITMENTS",
                "ACCOUNTS",
                "BALANCES",
                "CONTRACTS",
                "ACCOUNT_PLANS",
                "AGGREGATIONS",
                "PLANS",
                "PRICING",
                "PRICING_BANDS",
                "BILL_LINE_ITEMS",
                "METERS",
                "PRODUCTS",
                "COMPOUND_AGGREGATIONS",
                "PLAN_GROUPS",
                "PLAN_GROUP_LINKS",
                "PLAN_TEMPLATES",
                "BALANCE_TRANSACTIONS",
            ]
        ],
        source_type: Literal["USAGE", "OPERATIONAL"],
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ScheduleCreateResponse:
        """Create a new Data Export Schedule.

        Each Schedule can be configured for exporting
        _only one_ of either Usage or Operational data:

        **Operational Data Exports**.

        - Use the `operationalDataTypes` parameter to specify the entities whose
          operational data you want to include in the export each time the Export
          Schedule runs.
        - For each of the entity types you select, each time the Export Schedule runs a
          separate file is compiled containing the operational data for all entities of
          that type that exist in your Organization

        **Usage Data Exports**.

        - Select the Meters and Accounts whose usage data you want to include in the
          export each time the Export Schedule runs.
        - If _don't want to aggregate_ the usage data collected by the selected Meters,
          use **ORIGINAL** for `aggregationFrequency`, which is the _default_. This
          means the raw usage data collected by any type of Data Fields and the values
          for any Derived Fields on the selected Meters will be included in the export.
        - If you _do want to aggregate_ the usage data collected by the selected Meters,
          use one of the other options for `aggregationFrequency`: **HOUR**, **DAY**,
          **WEEK**, or **MONTH**. You _must_ then also specified an `aggregation` method
          to be used on the usage data before export. Importantly, if you do aggregate
          usage data, only the usage data collected by any numeric Data Fields on the
          selected Meters - those of type **MEASURE**, **INCOME**, or **COST** - will be
          included in the export each time the Export Schedule runs.

        Args:
          operational_data_types: A list of the entities whose operational data is included in the data export.

          source_type

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        org_id: str | None = None,
        aggregation_frequency: Literal["ORIGINAL", "HOUR", "DAY", "WEEK", "MONTH"],
        source_type: Literal["USAGE", "OPERATIONAL"],
        time_period: Literal[
            "TODAY",
            "YESTERDAY",
            "WEEK_TO_DATE",
            "CURRENT_MONTH",
            "LAST_30_DAYS",
            "LAST_35_DAYS",
            "PREVIOUS_WEEK",
            "PREVIOUS_MONTH",
        ],
        account_ids: List[str] | NotGiven = NOT_GIVEN,
        aggregation: Literal["SUM", "MIN", "MAX", "COUNT", "LATEST", "MEAN"] | NotGiven = NOT_GIVEN,
        meter_ids: List[str] | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ScheduleCreateResponse:
        """Create a new Data Export Schedule.

        Each Schedule can be configured for exporting
        _only one_ of either Usage or Operational data:

        **Operational Data Exports**.

        - Use the `operationalDataTypes` parameter to specify the entities whose
          operational data you want to include in the export each time the Export
          Schedule runs.
        - For each of the entity types you select, each time the Export Schedule runs a
          separate file is compiled containing the operational data for all entities of
          that type that exist in your Organization

        **Usage Data Exports**.

        - Select the Meters and Accounts whose usage data you want to include in the
          export each time the Export Schedule runs.
        - If _don't want to aggregate_ the usage data collected by the selected Meters,
          use **ORIGINAL** for `aggregationFrequency`, which is the _default_. This
          means the raw usage data collected by any type of Data Fields and the values
          for any Derived Fields on the selected Meters will be included in the export.
        - If you _do want to aggregate_ the usage data collected by the selected Meters,
          use one of the other options for `aggregationFrequency`: **HOUR**, **DAY**,
          **WEEK**, or **MONTH**. You _must_ then also specified an `aggregation` method
          to be used on the usage data before export. Importantly, if you do aggregate
          usage data, only the usage data collected by any numeric Data Fields on the
          selected Meters - those of type **MEASURE**, **INCOME**, or **COST** - will be
          included in the export each time the Export Schedule runs.

        Args:
          aggregation_frequency: Specifies the time period for the aggregation of usage data included each time
              the Data Export Schedule runs:

              - **ORIGINAL**. Usage data is _not aggregated_. If you select to not aggregate,
                then raw usage data measurements collected by all Data Field types and any
                Derived Fields on the selected Meters are included in the export. This is the
                _Default_.

              If you want to aggregate usage data for the Export Schedule you must define an
              `aggregationFrequency`:

              - **HOUR**. Aggregated hourly.
              - **DAY**. Aggregated daily.
              - **WEEK**. Aggregated weekly.
              - **MONTH**. Aggregated monthly.

              - If you select to aggregate usage data for a Export Schedule, then only the
                aggregated usage data collected by numeric Data Fields of type **MEASURE**,
                **INCOME**, or **COST** on selected Meters are included in the export.

              **NOTE**: If you define an `aggregationFrequency` other than **ORIGINAL** and do
              not define an `aggregation` method, then you'll receive and error.

          source_type

          time_period: Define a time period to control the range of usage data you want the data export
              to contain when it runs:

              - **TODAY**. Data collected for the current day up until the time the export
                runs.
              - **YESTERDAY**. Data collected for the day before the export runs - that is,
                the 24 hour period from midnight to midnight of the day before.
              - **WEEK_TO_DATE**. Data collected for the period covering the current week to
                the date and time the export runs, and weeks run Monday to Monday.
              - **CURRENT_MONTH**. Data collected for the current month in which the export is
                ran up to and including the date and time the export runs.
              - **LAST_30_DAYS**. Data collected for the 30 days prior to the date the export
                is ran.
              - **LAST_35_DAYS**. Data collected for the 35 days prior to the date the export
                is ran.
              - **PREVIOUS_WEEK**. Data collected for the previous full week period, and weeks
                run Monday to Monday.
              - **PREVIOUS_MONTH**. Data collected for the previous full month period.

              For more details and examples, see the
              [Time Period](https://www.m3ter.com/docs/guides/data-exports/creating-export-schedules#time-period)
              section in our main User Documentation.

          account_ids: List of account IDs for which the usage data will be exported.

          aggregation: Specifies the aggregation method applied to usage data collected in the numeric
              Data Fields of Meters included for the Data Export Schedule - that is, Data
              Fields of type **MEASURE**, **INCOME**, or **COST**:

              - **SUM**. Adds the values.
              - **MIN**. Uses the minimum value.
              - **MAX**. Uses the maximum value.
              - **COUNT**. Counts the number of values.
              - **LATEST**. Uses the most recent value. Note: Based on the timestamp `ts`
                value of usage data measurement submissions. If using this method, please
                ensure _distinct_ `ts` values are used for usage data measurement submissions.

          meter_ids: List of meter IDs for which the usage data will be exported.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["operational_data_types", "source_type"], ["aggregation_frequency", "source_type", "time_period"])
    async def create(
        self,
        *,
        org_id: str | None = None,
        operational_data_types: List[
            Literal[
                "BILLS",
                "COMMITMENTS",
                "ACCOUNTS",
                "BALANCES",
                "CONTRACTS",
                "ACCOUNT_PLANS",
                "AGGREGATIONS",
                "PLANS",
                "PRICING",
                "PRICING_BANDS",
                "BILL_LINE_ITEMS",
                "METERS",
                "PRODUCTS",
                "COMPOUND_AGGREGATIONS",
                "PLAN_GROUPS",
                "PLAN_GROUP_LINKS",
                "PLAN_TEMPLATES",
                "BALANCE_TRANSACTIONS",
            ]
        ]
        | NotGiven = NOT_GIVEN,
        source_type: Literal["USAGE", "OPERATIONAL"],
        version: int | NotGiven = NOT_GIVEN,
        aggregation_frequency: Literal["ORIGINAL", "HOUR", "DAY", "WEEK", "MONTH"] | NotGiven = NOT_GIVEN,
        time_period: Literal[
            "TODAY",
            "YESTERDAY",
            "WEEK_TO_DATE",
            "CURRENT_MONTH",
            "LAST_30_DAYS",
            "LAST_35_DAYS",
            "PREVIOUS_WEEK",
            "PREVIOUS_MONTH",
        ]
        | NotGiven = NOT_GIVEN,
        account_ids: List[str] | NotGiven = NOT_GIVEN,
        aggregation: Literal["SUM", "MIN", "MAX", "COUNT", "LATEST", "MEAN"] | NotGiven = NOT_GIVEN,
        meter_ids: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ScheduleCreateResponse:
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return cast(
            ScheduleCreateResponse,
            await self._post(
                f"/organizations/{org_id}/dataexports/schedules",
                body=await async_maybe_transform(
                    {
                        "operational_data_types": operational_data_types,
                        "source_type": source_type,
                        "version": version,
                        "aggregation_frequency": aggregation_frequency,
                        "time_period": time_period,
                        "account_ids": account_ids,
                        "aggregation": aggregation,
                        "meter_ids": meter_ids,
                    },
                    schedule_create_params.ScheduleCreateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, ScheduleCreateResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ScheduleRetrieveResponse:
        """Retrieve a Data Export Schedule for the given UUID.

        Each Schedule can be
        configured for exporting _only one_ of either Usage or Operational data.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return cast(
            ScheduleRetrieveResponse,
            await self._get(
                f"/organizations/{org_id}/dataexports/schedules/{id}",
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, ScheduleRetrieveResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    @overload
    async def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        operational_data_types: List[
            Literal[
                "BILLS",
                "COMMITMENTS",
                "ACCOUNTS",
                "BALANCES",
                "CONTRACTS",
                "ACCOUNT_PLANS",
                "AGGREGATIONS",
                "PLANS",
                "PRICING",
                "PRICING_BANDS",
                "BILL_LINE_ITEMS",
                "METERS",
                "PRODUCTS",
                "COMPOUND_AGGREGATIONS",
                "PLAN_GROUPS",
                "PLAN_GROUP_LINKS",
                "PLAN_TEMPLATES",
                "BALANCE_TRANSACTIONS",
            ]
        ],
        source_type: Literal["USAGE", "OPERATIONAL"],
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ScheduleUpdateResponse:
        """Update a Data Export Schedule for the given UUID.

        Each Schedule can be
        configured for exporting _only one_ of either Usage or Operational data:

        **Operational Data Exports**.

        - Use the `operationalDataTypes` parameter to specify the entities whose
          operational data you want to include in the export each time the Export
          Schedule runs.
        - For each of the entity types you select, each time the Export Schedule runs a
          separate file is compiled containing the operational data for all entities of
          that type that exist in your Organization

        **Usage Data Exports**.

        - Select the Meters and Accounts whose usage data you want to include in the
          export each time the Export Schedule runs.
        - If _don't want to aggregate_ the usage data collected by the selected Meters,
          use **ORIGINAL** for `aggregationFrequency`, which is the _default_. This
          means the raw usage data collected by any type of Data Fields and the values
          for any Derived Fields on the selected Meters will be included in the export.
        - If you _do want to aggregate_ the usage data collected by the selected Meters,
          use one of the other options for `aggregationFrequency`: **HOUR**, **DAY**,
          **WEEK**, or **MONTH**. You _must_ then also specified an `aggregation` method
          to be used on the usage data before export. Importantly, if you do aggregate
          usage data, only the usage data collected by any numeric Data Fields on the
          selected Meters - those of type **MEASURE**, **INCOME**, or **COST** - will be
          included in the export each time the Export Schedule runs.

        Args:
          operational_data_types: A list of the entities whose operational data is included in the data export.

          source_type

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        aggregation_frequency: Literal["ORIGINAL", "HOUR", "DAY", "WEEK", "MONTH"],
        source_type: Literal["USAGE", "OPERATIONAL"],
        time_period: Literal[
            "TODAY",
            "YESTERDAY",
            "WEEK_TO_DATE",
            "CURRENT_MONTH",
            "LAST_30_DAYS",
            "LAST_35_DAYS",
            "PREVIOUS_WEEK",
            "PREVIOUS_MONTH",
        ],
        account_ids: List[str] | NotGiven = NOT_GIVEN,
        aggregation: Literal["SUM", "MIN", "MAX", "COUNT", "LATEST", "MEAN"] | NotGiven = NOT_GIVEN,
        meter_ids: List[str] | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ScheduleUpdateResponse:
        """Update a Data Export Schedule for the given UUID.

        Each Schedule can be
        configured for exporting _only one_ of either Usage or Operational data:

        **Operational Data Exports**.

        - Use the `operationalDataTypes` parameter to specify the entities whose
          operational data you want to include in the export each time the Export
          Schedule runs.
        - For each of the entity types you select, each time the Export Schedule runs a
          separate file is compiled containing the operational data for all entities of
          that type that exist in your Organization

        **Usage Data Exports**.

        - Select the Meters and Accounts whose usage data you want to include in the
          export each time the Export Schedule runs.
        - If _don't want to aggregate_ the usage data collected by the selected Meters,
          use **ORIGINAL** for `aggregationFrequency`, which is the _default_. This
          means the raw usage data collected by any type of Data Fields and the values
          for any Derived Fields on the selected Meters will be included in the export.
        - If you _do want to aggregate_ the usage data collected by the selected Meters,
          use one of the other options for `aggregationFrequency`: **HOUR**, **DAY**,
          **WEEK**, or **MONTH**. You _must_ then also specified an `aggregation` method
          to be used on the usage data before export. Importantly, if you do aggregate
          usage data, only the usage data collected by any numeric Data Fields on the
          selected Meters - those of type **MEASURE**, **INCOME**, or **COST** - will be
          included in the export each time the Export Schedule runs.

        Args:
          aggregation_frequency: Specifies the time period for the aggregation of usage data included each time
              the Data Export Schedule runs:

              - **ORIGINAL**. Usage data is _not aggregated_. If you select to not aggregate,
                then raw usage data measurements collected by all Data Field types and any
                Derived Fields on the selected Meters are included in the export. This is the
                _Default_.

              If you want to aggregate usage data for the Export Schedule you must define an
              `aggregationFrequency`:

              - **HOUR**. Aggregated hourly.
              - **DAY**. Aggregated daily.
              - **WEEK**. Aggregated weekly.
              - **MONTH**. Aggregated monthly.

              - If you select to aggregate usage data for a Export Schedule, then only the
                aggregated usage data collected by numeric Data Fields of type **MEASURE**,
                **INCOME**, or **COST** on selected Meters are included in the export.

              **NOTE**: If you define an `aggregationFrequency` other than **ORIGINAL** and do
              not define an `aggregation` method, then you'll receive and error.

          source_type

          time_period: Define a time period to control the range of usage data you want the data export
              to contain when it runs:

              - **TODAY**. Data collected for the current day up until the time the export
                runs.
              - **YESTERDAY**. Data collected for the day before the export runs - that is,
                the 24 hour period from midnight to midnight of the day before.
              - **WEEK_TO_DATE**. Data collected for the period covering the current week to
                the date and time the export runs, and weeks run Monday to Monday.
              - **CURRENT_MONTH**. Data collected for the current month in which the export is
                ran up to and including the date and time the export runs.
              - **LAST_30_DAYS**. Data collected for the 30 days prior to the date the export
                is ran.
              - **LAST_35_DAYS**. Data collected for the 35 days prior to the date the export
                is ran.
              - **PREVIOUS_WEEK**. Data collected for the previous full week period, and weeks
                run Monday to Monday.
              - **PREVIOUS_MONTH**. Data collected for the previous full month period.

              For more details and examples, see the
              [Time Period](https://www.m3ter.com/docs/guides/data-exports/creating-export-schedules#time-period)
              section in our main User Documentation.

          account_ids: List of account IDs for which the usage data will be exported.

          aggregation: Specifies the aggregation method applied to usage data collected in the numeric
              Data Fields of Meters included for the Data Export Schedule - that is, Data
              Fields of type **MEASURE**, **INCOME**, or **COST**:

              - **SUM**. Adds the values.
              - **MIN**. Uses the minimum value.
              - **MAX**. Uses the maximum value.
              - **COUNT**. Counts the number of values.
              - **LATEST**. Uses the most recent value. Note: Based on the timestamp `ts`
                value of usage data measurement submissions. If using this method, please
                ensure _distinct_ `ts` values are used for usage data measurement submissions.

          meter_ids: List of meter IDs for which the usage data will be exported.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["operational_data_types", "source_type"], ["aggregation_frequency", "source_type", "time_period"])
    async def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        operational_data_types: List[
            Literal[
                "BILLS",
                "COMMITMENTS",
                "ACCOUNTS",
                "BALANCES",
                "CONTRACTS",
                "ACCOUNT_PLANS",
                "AGGREGATIONS",
                "PLANS",
                "PRICING",
                "PRICING_BANDS",
                "BILL_LINE_ITEMS",
                "METERS",
                "PRODUCTS",
                "COMPOUND_AGGREGATIONS",
                "PLAN_GROUPS",
                "PLAN_GROUP_LINKS",
                "PLAN_TEMPLATES",
                "BALANCE_TRANSACTIONS",
            ]
        ]
        | NotGiven = NOT_GIVEN,
        source_type: Literal["USAGE", "OPERATIONAL"],
        version: int | NotGiven = NOT_GIVEN,
        aggregation_frequency: Literal["ORIGINAL", "HOUR", "DAY", "WEEK", "MONTH"] | NotGiven = NOT_GIVEN,
        time_period: Literal[
            "TODAY",
            "YESTERDAY",
            "WEEK_TO_DATE",
            "CURRENT_MONTH",
            "LAST_30_DAYS",
            "LAST_35_DAYS",
            "PREVIOUS_WEEK",
            "PREVIOUS_MONTH",
        ]
        | NotGiven = NOT_GIVEN,
        account_ids: List[str] | NotGiven = NOT_GIVEN,
        aggregation: Literal["SUM", "MIN", "MAX", "COUNT", "LATEST", "MEAN"] | NotGiven = NOT_GIVEN,
        meter_ids: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ScheduleUpdateResponse:
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return cast(
            ScheduleUpdateResponse,
            await self._put(
                f"/organizations/{org_id}/dataexports/schedules/{id}",
                body=await async_maybe_transform(
                    {
                        "operational_data_types": operational_data_types,
                        "source_type": source_type,
                        "version": version,
                        "aggregation_frequency": aggregation_frequency,
                        "time_period": time_period,
                        "account_ids": account_ids,
                        "aggregation": aggregation,
                        "meter_ids": meter_ids,
                    },
                    schedule_update_params.ScheduleUpdateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, ScheduleUpdateResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        ids: List[str] | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[ScheduleListResponse, AsyncCursor[ScheduleListResponse]]:
        """Retrieve a list of Data Export Schedules created for your Organization.

        You can
        filter the response by Schedules `ids`.

        The response will contain an array for both the operational and usage Data
        Export Schedules in your Organization.

        Args:
          ids: Data Export Schedule IDs to filter the returned list by.

          next_token: `nextToken` for multi page retrievals

          page_size: Number of schedules to retrieve per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/dataexports/schedules",
            page=AsyncCursor[ScheduleListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    schedule_list_params.ScheduleListParams,
                ),
            ),
            model=ScheduleListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ScheduleDeleteResponse:
        """Delete the Data Export Schedule for the given UUID.

        Each Schedule can be
        configured for exporting _only one_ of either Usage or Operational data.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return cast(
            ScheduleDeleteResponse,
            await self._delete(
                f"/organizations/{org_id}/dataexports/schedules/{id}",
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, ScheduleDeleteResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class SchedulesResourceWithRawResponse:
    def __init__(self, schedules: SchedulesResource) -> None:
        self._schedules = schedules

        self.create = to_raw_response_wrapper(
            schedules.create,
        )
        self.retrieve = to_raw_response_wrapper(
            schedules.retrieve,
        )
        self.update = to_raw_response_wrapper(
            schedules.update,
        )
        self.list = to_raw_response_wrapper(
            schedules.list,
        )
        self.delete = to_raw_response_wrapper(
            schedules.delete,
        )


class AsyncSchedulesResourceWithRawResponse:
    def __init__(self, schedules: AsyncSchedulesResource) -> None:
        self._schedules = schedules

        self.create = async_to_raw_response_wrapper(
            schedules.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            schedules.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            schedules.update,
        )
        self.list = async_to_raw_response_wrapper(
            schedules.list,
        )
        self.delete = async_to_raw_response_wrapper(
            schedules.delete,
        )


class SchedulesResourceWithStreamingResponse:
    def __init__(self, schedules: SchedulesResource) -> None:
        self._schedules = schedules

        self.create = to_streamed_response_wrapper(
            schedules.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            schedules.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            schedules.update,
        )
        self.list = to_streamed_response_wrapper(
            schedules.list,
        )
        self.delete = to_streamed_response_wrapper(
            schedules.delete,
        )


class AsyncSchedulesResourceWithStreamingResponse:
    def __init__(self, schedules: AsyncSchedulesResource) -> None:
        self._schedules = schedules

        self.create = async_to_streamed_response_wrapper(
            schedules.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            schedules.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            schedules.update,
        )
        self.list = async_to_streamed_response_wrapper(
            schedules.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            schedules.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["DebitLineItemResponse"]


class DebitLineItemResponse(BaseModel):
    id: str
    """The UUID of the entity."""

    amount: float
    """The amount for the line item."""

    description: str
    """The description of the line item."""

    product_id: str = FieldInfo(alias="productId")
    """The UUID of the Product."""

    referenced_bill_id: str = FieldInfo(alias="referencedBillId")
    """The UUID of the bill for the line item."""

    referenced_line_item_id: str = FieldInfo(alias="referencedLineItemId")
    """The UUID of the line item."""

    service_period_end_date: datetime = FieldInfo(alias="servicePeriodEndDate")
    """The service period end date in ISO-8601 format.

    _(exclusive of the ending date)_.
    """

    service_period_start_date: datetime = FieldInfo(alias="servicePeriodStartDate")
    """The service period start date in ISO-8601 format.

    _(inclusive of the starting date)_.
    """

    version: int
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The id of the user who created this debit line item."""

    debit_reason_id: Optional[str] = FieldInfo(alias="debitReasonId", default=None)
    """The UUID of the debit reason for this debit line item."""

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """The DateTime when the debit line item was created _(in ISO-8601 format)_."""

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """The DateTime when the debit line item was last modified _(in ISO-8601 format)_."""

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """The id of the user who last modified this debit line item."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["ScheduledEventConfigurationResponse"]


class ScheduledEventConfigurationResponse(BaseModel):
    id: str
    """The UUID of the entity."""

    entity: str
    """
    The referenced configuration or billing entity for which the desired scheduled
    Event will trigger.
    """

    field: str
    """
    A DateTime field for which the desired scheduled Event will trigger - this must
    be a DateTime field on the referenced billing or configuration entity.
    """

    name: str
    """The name of the custom Scheduled Event Configuration."""

    offset: int
    """
    The offset in days from the specified DateTime field on the referenced entity
    when the scheduled Event will trigger.
    """

    version: int
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The ID of the user who created this item."""

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """The DateTime when this item was created _(in ISO-8601 format)_."""

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """The DateTime when this item was last modified _(in ISO-8601 format)_."""

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """The ID of the user who last modified this item."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, TypedDict

__all__ = ["PermissionStatementResponseParam"]


class PermissionStatementResponseParam(TypedDict, total=False):
    action: Required[
        List[
            Literal[
                "ALL",
                "CONFIG_CREATE",
                "CONFIG_RETRIEVE",
                "CONFIG_UPDATE",
                "CONFIG_DELETE",
                "CONFIG_EXPORT",
                "ANALYTICS_QUERY",
                "MEASUREMENTS_UPLOAD",
                "MEASUREMENTS_FILEUPLOAD",
                "MEASUREMENTS_RETRIEVE",
                "MEASUREMENTS_EXPORT",
                "FORECAST_RETRIEVE",
                "HEALTHSCORES_RETRIEVE",
                "ANOMALIES_RETRIEVE",
                "EXPORTS_DOWNLOAD",
            ]
        ]
    ]
    """
    The actions available to users who are assigned the Permission Policy - what
    they can do or cannot do with respect to the specified resource.

    **NOTE:** Use lower case and a colon-separated format, for example, if you want
    to confer full CRUD, use:

    ```
    "config:create",
    "config:delete",
    "config:retrieve",
    "config:update"
    ```
    """

    effect: Required[Literal["ALLOW", "DENY"]]
    """
    Specifies whether or not the user is allowed to perform the action on the
    resource.

    **NOTE:** Use lower case, for example: `"allow"`. If you use upper case, you'll
    receive an error.
    """

    resource: Required[List[str]]
    """
    See
    [Statements - Available Resources](https://www.m3ter.com/docs/guides/managing-organization-and-users/creating-and-managing-permissions#statements---available-resources)
    for a listing of available resources for Permission Policy statements.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .m3ter_signed_credentials_response import M3terSignedCredentialsResponse

__all__ = ["Webhook"]


class Webhook(BaseModel):
    id: str
    """The UUID of the entity."""

    version: int
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """

    active: Optional[bool] = None

    code: Optional[str] = None

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The ID of the user who created this item."""

    credentials: Optional[M3terSignedCredentialsResponse] = None
    """Response representing a set of credentials used for signing m3ter requests."""

    description: Optional[str] = None

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """The DateTime when this item was created _(in ISO-8601 format)_."""

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """The DateTime when this item was last modified _(in ISO-8601 format)_."""

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """The ID of the user who last modified this item."""

    name: Optional[str] = None

    url: Optional[str] = None
    """The URL to which webhook requests are sent."""
